from apscheduler.schedulers.blocking import BlockingScheduler
# from datetime import datetime


def scheduler(my_job, run_datetime, args_=None):
    """

    :param args_:
    :param my_job:
    :param run_datetime: datetime(2021, 8, 26, 11, 30, 5)
    :return:
    """
    sched = BlockingScheduler()
    sched.add_job(my_job, 'date', run_date=run_datetime, args=args_)
    sched.start()



import os
import cv2
import shutil
from tqdm import tqdm
import random
from .utils import make_random_name
from pathlib import Path
# import imagededup
# from imagededup.methods import PHash
# from imagededup.methods import CNN


class FolderTools:
    def __init__(self, root_path, save_path=None):
        self.root_path = root_path
        self.save_path = save_path
        if save_path is not None:
            os.makedirs(self.save_path, exist_ok=True)

    # def find_duplicates(self):
    #     cnn_encoder = CNN()
    #     duplicates_cnn = cnn_encoder.find_duplicates(image_dir=self.root_path, scores=True)
    #     # print(len(duplicates_cnn))
    #     already_name = []
    #     for k, v in tqdm(duplicates_cnn.items()):
    #         file_name = k
    #         if file_name not in already_name:
    #             shutil.copyfile(os.path.join(self.root_path, file_name), os.path.join(self.save_path, file_name))
    #         already_name.append(file_name)
    #         for vv in v:
    #             already_name.append(vv[0])
    #         already_name = list(set(already_name))

    def rename_folder_files(self, prefix1, prefix2):
        for f in tqdm(os.listdir(self.root_path)):
            f_p = os.path.join(self.root_path, f)
            aim_p = os.path.join(self.root_path, f.replace(prefix1, prefix2))
            os.rename(f_p, aim_p)

    def extract_imgs_from_folder(self, img_num=0, patten=None, random_flag=True):
        if patten is None:
            patten = '*.jpg'
        count = 0
        path = Path(self.root_path)
        file_list = list(path.rglob(patten))
        if random_flag:
            random.shuffle(file_list)
        for f_p in file_list:
            # print(f_p)
            img = cv2.imread(str(f_p))
            if img is not None:
                ff = f_p.name
                aim_p = os.path.join(self.save_path, ff)
                if os.path.exists(aim_p):
                    aim_p = os.path.join(self.save_path, make_random_name(ff))
                shutil.copy(f_p, aim_p)
                count += 1
                if count % 1000 == 0:
                    print('copy {} imgs from {} to {}'.format(count, self.root_path, self.save_path))
                if count == img_num:
                    return

    def clean_one_folder_from_another(self, ):
        # clean root_p from save_p
        f_list = os.listdir(self.root_path)
        f_list_to_do = os.listdir(self.save_path)

        f_list_remove = list(set(f_list).intersection(set(f_list_to_do)))
        # print(len(f_list_remove))
        for f in tqdm(f_list_remove):
            os.remove(os.path.join(self.root_path, f))



import os
import shutil
from tqdm import tqdm
from utils import os_call


def get_shell(dir_path, shell_name):
    os_call('cp ./shell_scripts/{}.sh \'{}\''.format(shell_name, dir_path))
    # shutil.copyfile('./shell_scripts/uncompress.sh', file_path + '/uncompress.sh')


def destroy_shell(dir_path, shell_name):
    os.remove('{}/{}.sh'.format(dir_path, shell_name))


def batch_uncompress(dir_path, pwd, pwd_flag=None):

    for f in os.listdir(dir_path):
        f_p = os.path.join(dir_path, f)
        if os.path.isdir(f_p):
            if f_p.find(pwd_flag) > 0:
                get_shell(f_p, 'uncompress')
                os_call('cd {} && sh uncompress.sh 7z ./ 202166'.format(f_p))
                destroy_shell(f_p, 'uncompress')
            else:
                get_shell(f_p, 'uncompress')
                os_call('cd {} && sh uncompress.sh tar ./'.format(f_p))
                destroy_shell(f_p, 'uncompress')


if __name__ == '__main__':
    p = ''
    # get_shell(p, 'uncompress')
    # destroy_shell(p, 'uncompress')
    batch_uncompress(p, '',pwd_flag='密码')


import os
import re

import cv2
import shutil
from tqdm import tqdm
from .utils import os_call
import numpy as np
from moviepy.editor import VideoFileClip


def decode_fourcc(cc):
    return "".join([chr((int(cc) >> 8 * i) & 0xFF) for i in range(4)])


class VideoTools:
    def __init__(self, video_p, verbose=True):
        self.video_path = video_p
        self.video_dir, self.video_name = os.path.split(self.video_path)
        self.prefix, self.suffix = os.path.splitext(self.video_name)
        if verbose:
            self.print_video_info()

    def print_video_info(self):

        # 获得视频的格式
        cap = cv2.VideoCapture(self.video_path)
        fourcc = cap.get(cv2.CAP_PROP_FOURCC)
        # 获得码率及尺寸
        fps = cap.get(cv2.CAP_PROP_FPS)
        frame_number = cap.get(cv2.CAP_PROP_FRAME_COUNT)  # 视频文件的帧数
        duration = frame_number / fps  # 帧速率/视频总帧数 是s

        size = (int(cap.get(cv2.CAP_PROP_FRAME_WIDTH)),
                int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT)))
        cap.release()
        print(
            'video info:\nname: {}\nfourcc: {}\nfps: {}\nsize: {}'.format(self.video_name, decode_fourcc(fourcc), fps,
                                                                          size))

    def show_video_cv(self):
        cap = cv2.VideoCapture(self.video_path)
        success = True
        while success:
            success, frame = cap.read()
            cv2.namedWindow("First Frame", 0)
            cv2.imshow('First Frame', frame)
            # cv2.waitKey(99999)
            if cv2.waitKey(100) == 27 or 0xFF == ord('q'):
                break
        cap.release()
        cv2.destroyAllWindows()

    def video_2_h264(self, inplace=True):
        if not inplace:
            os_call('ffmpeg -i {} -vcodec h264 {}'.format(self.video_path,
                                                          self.video_path.replace(self.suffix, '_h264_out.mp4')))
        else:
            temp_p = self.video_path.replace(self.suffix, '_temp_copy.mp4')
            video_path_new = self.video_path.replace(self.suffix, '.mp4')
            os_call(
                'mv {} {} && ffmpeg -i {} -vcodec h264 {} && rm {}'.format(self.video_path, temp_p, temp_p,
                                                                           video_path_new, temp_p))

    def change_video_speed(self, speed=1):
        assert 0.5 <= speed <= 2.0, 'Speed must between 0.5-2.0 .'
        command = 'ffmpeg -i {} -filter_complex "[0:v]setpts={}*PTS[v];[0:a]atempo={}[a]" -map "[v]" -map "[a]" {}.mp4'.format(
            self.video_path, 1 / speed, speed, self.video_dir + '/' + self.prefix + '_speed_out')
        os_call(command)

    @staticmethod
    def concat_multi_video(video_dir):
        file_list = []
        for video_n in os.listdir(video_dir):
            file_list.append('file \'{}\'\n'.format(video_n))
        filelist_p = video_dir + '/filelist.txt'
        with open(filelist_p, 'w') as f:
            f.writelines(file_list)
        command = 'ffmpeg -f concat -i {} -c copy {}.mp4 && rm {}'.format(filelist_p,
                                                                          video_dir + '/' + 'multi_video_concat_result',
                                                                          filelist_p)
        os_call(command)

    def crop_video(self, rec_list: tuple, format='libx264'):
        """
        out_w is the width of the output rectangle
        out_h is the height of the output rectangle
        x and y specify the top left corner of the output rectangle
        """
        x, y, out_w, out_h = rec_list
        size_str = '{}:{}:{}:{}'.format(str(out_w), str(out_h), str(x), str(y))
        command = 'ffmpeg -y -i {} -filter:v "crop={}" -c:v {} -crf 17 -c:a copy {}.mp4'.format(self.video_path,
                                                                                                size_str, format,
                                                                                                self.video_dir + '/' + self.prefix + '_out')
        os_call(command)

    def cut_video(self, start, last_time, accurate=False):
        assert re.match(r"(\d{1,2}:\d{1,2}:\d{1,2})",
                        start) is not None, 'The time format: start:00:00:15 last_time:00:00:15 etc.'
        assert re.match(r"(\d{1,2}:\d{1,2}:\d{1,2})",
                        last_time) is not None, 'The time format: start:00:00:15 last_time:00:00:15 etc.'
        if not accurate:
            command = 'ffmpeg -y -ss {} -t {} -i {} -codec copy {}.mp4'.format(start, last_time, self.video_path,
                                                                               self.video_dir + '/' + self.prefix + '_cut_out')
        else:
            command = 'ffmpeg -y -ss {} -t {} -i {} {}.mp4'.format(start, last_time, self.video_path,
                                                                   self.video_dir + '/' + self.prefix + '_cut_out')
        os_call(command)

    def add_text(self, text, left_top_coord: tuple, fontsize=20):

        command = 'ffmpeg -i {} -vf drawtext="text={}:x={}:y={}:fontsize={}:fontcolor=white:box=1:boxcolor=blue" -y {}.mp4'.format(
            self.video_path, text, left_top_coord[0], left_top_coord[1], fontsize,
            self.video_dir + '/' + self.prefix + '_add_text_out')
        os_call(command)

    def reverse_crop_video(self, vp_overlay, rec_list: tuple, format='libx264rgb'):
        """
        x and y specify the top left corner of the output rectangle
        """
        x, y = rec_list
        size_str = '{}:{}'.format(str(x), str(y))
        # libx265 libx264 libx264rgb
        # command = 'ffmpeg -y -i {} -i {} -filter_complex overlay={} -c:v {} -crf 17 -c:a copy {}.mp4'.format(
        command = 'ffmpeg -y -i {} -i {} -filter_complex overlay={} -c:v {} -c:a copy {}.mp4'.format(
            self.video_path, vp_overlay, size_str, format,
            self.video_dir + '/' + self.prefix + '_reverse_out')
        os_call(command)

    def video_2_frame(self, interval=1, out_path=None):
        if out_path is None:
            save_path = self.video_path.split('.mp4')[0] + '/'
        else:
            save_path = out_path

        is_exists = os.path.exists(save_path)
        if not is_exists:
            os.makedirs(save_path)
            print('path of %s is build' % save_path)
        else:
            shutil.rmtree(save_path)
            os.makedirs(save_path)
            print('path of %s already exist and rebuild' % save_path)

        # 开始读视频
        video_capture = cv2.VideoCapture(self.video_path)
        i = 0
        j = 0

        while True:
            success, frame = video_capture.read()
            i += 1
            # print(frame)
            if not success:
                print('done!')
                break
            if i % interval == 0:
                # 保存图片
                j += 1
                save_name = save_path + str(j) + '_' + str(i) + '.png'
                cv2.imwrite(save_name, frame)
                print('image of %s is saved' % save_name)
        video_capture.release()

    def resize_video(self, out_size=(768, 1024), inplace=False):

        out_p = self.video_path.replace(self.extension, '_{}x{}.mp4'.format(out_size[0], out_size[1]))
        if inplace:
            os.system('mv {} {}'.format(self.video_path, out_p))
            self.video_path, out_p = out_p, self.video_path

        cap = cv2.VideoCapture(self.video_path)
        fps = cap.get(cv2.CAP_PROP_FPS)

        video_writer = cv2.VideoWriter(out_p, cv2.VideoWriter_fourcc(*'mp4v'), fps, out_size)

        s = True
        while s:
            s, img1 = cap.read()
            img = cv2.resize(img1, out_size, cv2.INTER_LINEAR)
            video_writer.write(img)
        cap.release()

        if inplace:
            os.system('rm {} &'.format(self.video_path))

    def video_concat(self, video_path_2, concat_mode=None, copy_audio=True):
        if concat_mode is None:
            print('Need name concat_mode to \'vstack\' or \'hstack\' !')
            return
        img = None
        reader1 = cv2.VideoCapture(self.video_path)
        fps = reader1.get(cv2.CAP_PROP_FPS)
        video1_pre_path, video1_suffix = os.path.splitext(self.video_path)
        video_out_p = video1_pre_path + '_concat_out.mp4'
        if video1_suffix != '.mp4':
            print('Will output mp4 format video file !')
            print('Output path is {}'.format(video_out_p))

        reader2 = cv2.VideoCapture(video_path_2)
        width1 = int(reader1.get(cv2.CAP_PROP_FRAME_WIDTH))
        height1 = int(reader1.get(cv2.CAP_PROP_FRAME_HEIGHT))
        width2 = int(reader2.get(cv2.CAP_PROP_FRAME_WIDTH))
        height2 = int(reader2.get(cv2.CAP_PROP_FRAME_HEIGHT))

        if concat_mode == 'hstack':
            assert height1 == height2, 'height1: {} height2: {}'.format(height1, height2)
            output_size = (width1 + width2, height1)
        elif concat_mode == 'vstack':
            assert width1 == width2
            output_size = (width1, height1 + height2)
        else:
            output_size = (width1 + width2, height1)

        writer = cv2.VideoWriter(video_out_p,
                                 cv2.VideoWriter_fourcc(*'mp4v'),  # (*"mp4v") for mp4 output
                                 fps,  # fps
                                 output_size)  # resolution

        if not reader1.isOpened() or not reader2.isOpened():
            print('video1 read {}, video2 read {}'.format(reader1.isOpened(), reader2.isOpened()))
            return

        have_more_frame = True

        while have_more_frame:
            have_more_frame, frame1 = reader1.read()
            have_more_frame_2, frame2 = reader2.read()
            if not have_more_frame_2:
                break
            # frame1 = cv2.resize(frame1, (width, height))
            # frame2 = cv2.resize(frame2, (width, height))
            try:
                if concat_mode == 'hstack':
                    img = np.hstack((frame1, frame2))
                elif concat_mode == 'vstack':
                    img = np.vstack((frame1, frame2))
                else:
                    img = np.hstack((frame1, frame2))
            except ValueError:
                print('Got ValueError')
            writer.write(img)

        writer.release()
        reader1.release()
        reader2.release()

        if copy_audio:
            video = VideoFileClip(self.video_path)
            video_out = VideoFileClip(video_out_p)
            video_out_audio = video_out.set_audio(video.audio)
            video_out_audio.write_videofile(video_out_p.replace('_concat_out.mp4', '_concat_out_audio.mp4'))

        return video_out_p



import seaborn as sns
import numpy as np
import matplotlib.pyplot as plt


import os
n_list = []
for f in os.listdir(''):
    f_p = os.path.join('/workspace/151_cluster/nfs/dataset/deepfacelab/material/待处理/face512aligned', f)
    for ff in os.listdir(f_p):
        f_p_p = os.path.join(f_p, ff)
        f_p_len = len(os.listdir(f_p_p))
        # print(f_p_len)
        n_list.append(f_p_len)


sns.set_style("darkgrid")
# x = np.random.normal(size=200)
sns.displot(n_list, color='y')
# plt.plot(np.arange(10))
plt.show()


# VERSION 0.09
from .folder_tools import FolderTools
from .img_tools import ImgTools
from .video_tools import VideoTools
from .utils import MyTimer
from .cv_ops import CVImage
from .get_templates import GetTemplate



# -- coding: utf-8 --
# @Time : 2021/11/15
# @Author : ykk648
# @Project : https://github.com/ykk648/AI_power

from pynput import keyboard
from multiprocessing.dummy import Process, Queue, Array, Manager


class KeyboardListener(Process):
    def __init__(self, share_list):
        super().__init__()
        self.share_list = share_list
        # do your init

    def run(self):
        # Collect events until released
        with keyboard.Listener(
                on_press=self.on_press,
                on_release=self.on_release) as listener:
            listener.join()

    def on_press(self, key):
        try:
            print('alphanumeric key {0} pressed'.format(
                key.char))

        except AttributeError:
            print('special key {0} pressed'.format(key))

    @staticmethod
    def on_release(key):
        print('{0} released'.format(
            key))
        if key == keyboard.Key.esc:
            # Stop listener
            return False


if __name__ == '__main__':
    manager = Manager()
    share_list_ = manager.list()
    kl = KeyboardListener(share_list_)



import os
import uuid
import pickle
import shutil
import numpy as np
import time


def os_call(command):
    print(command)
    os.system(command)


def make_random_name(f_name):
    return uuid.uuid4().hex + '.' + f_name.split('.')[-1]


def flush_print(str_to_print):
    print("\r" + "{}".format(str_to_print), end="", flush=True)


def pickle_load(pickle_path):
    with open(pickle_path, 'rb') as f:
        dummy = pickle.load(f)
    return dummy


def get_my_dir():
    dir_name, filename = os.path.split(os.path.abspath(__file__))
    return dir_name


# def give_me_ai_power():
#     shutil.copytree('{}/../AI_power'.format(get_my_dir()), './AI_power')


def np_norm(x):
    return (x - np.average(x)) / np.std(x)


class MyTimer(object):
    """
    timer
    """

    def __enter__(self):
        self.t0 = time.time()

    def __exit__(self, exc_type, exc_val, exc_tb):
        print('[finished, spent time: {time:.2f}s]'.format(time=time.time() - self.t0))



from .utils import os_call, get_my_dir
import shutil
import os

tem_dict = {
    1: '{}/code_templates/multiprocess_consumer.py'.format(get_my_dir()),
    2: '{}/code_templates/multiprocess_by_concurrent.py'.format(get_my_dir()),
}


class GetTemplate:
    def __init__(self, t_serial=None):
        self.t = t_serial
        if self.t is None:
            print(tem_dict)
        else:
            self.copy_template()

    @staticmethod
    def show():
        help_msg = '1: multiprocess_consumer.py \n' \
                   '2: multiprocess_by_concurrent.py \n'
        print(help_msg)

    def copy_template(self):
        shutil.copyfile(tem_dict[self.t], os.getcwd())




# -- coding: utf-8 --
# @Time : 2021/11/19
# @Author : ykk648
# @Project : https://github.com/ykk648/cv2box
import cv2
import numpy as np
from PIL import Image
import base64
import io

"""
skimage and pillow read image based uint8 and RGB mode
opencv read image based uint8 and BGR mode
using opencv as the default image read method
"""


class CVImage:
    def __init__(self, image_in, image_format='cv2'):
        self.transform = None

        if isinstance(image_in, str) and image_format == 'cv2':
            # assert type(image_in) is str, 'if not give str path, name \'image_format\' !'
            self.cv_image = cv2.imread(image_in)
        elif 'cv' in image_format:
            self.cv_image = image_in
        elif 'pi' in image_format:
            self.cv_image = cv2.cvtColor(np.asarray(image_in), cv2.COLOR_RGB2BGR)
        elif 'sk' in image_format:
            self.cv_image = cv2.cvtColor((image_in * 255).astype(int), cv2.COLOR_RGB2BGR)
        elif 'ten' in image_format:
            image_numpy = image_in[0].cpu().float().numpy()
            self.cv_image = (np.transpose(image_numpy, (1, 2, 0)) + 1) / 2.0 * 255.0
        elif 'base' in image_format:
            img_data = base64.b64decode(image_in[22:])
            img_array = np.frombuffer(img_data, np.uint8)
            self.cv_image = cv2.imdecode(img_array, 1)
        elif 'byte' in image_format:
            self.cv_image = cv2.imdecode(np.frombuffer(io.BytesIO(image_in).read(), np.uint8), 1)
        else:
            raise 'Can not find image_format ！'

    @property
    def rgb(self):
        return cv2.cvtColor(self.cv_image, cv2.COLOR_BGR2RGB)

    @property
    def bgr(self):
        return self.cv_image

    @property
    def pillow(self):
        return Image.fromarray(cv2.cvtColor(self.cv_image, cv2.COLOR_BGR2RGB))

    @property
    def tensor(self):
        import torch
        assert self.transform is not None, 'Use set_transform first !'
        img = self.transform(self.cv_image)
        return torch.unsqueeze(img, 0)

    @property
    def base64(self):
        """
        :return: jpg format base64 code
        """
        image = cv2.imencode('.jpg', self.cv_image)[1]
        image_code = str(base64.b64encode(image))[2:-1]
        return 'data:image/jpg;base64,' + image_code

    def resize(self, size):
        if type(size) == tuple:
            self.cv_image = cv2.resize(self.cv_image, size)
        elif type(size) == int:
            self.cv_image = cv2.resize(self.cv_image, (size, size))
        else:
            raise 'Check the size input !'

    def set_transform(self, transform=None):
        from torchvision import transforms
        if not transform:
            self.transform = transform
        else:
            self.transform = transforms.Compose([
                transforms.ToTensor(),
                transforms.Normalize((0.5, 0.5, 0.5), (0.5, 0.5, 0.5)),
            ])

    def show(self):
        cv2.namedWindow('test', 0)
        cv2.imshow('test', self.cv_image)
        cv2.waitKey(0)

    def save(self, img_save_p):
        cv2.imwrite(img_save_p, self.cv_image)



import numpy as np
from .util import np_norm


class CalDistance:
    def __init__(self, vector1, vector2):
        self.vector1 = vector1
        self.vector2 = vector2

    def euc(self):
        return np.sqrt(np.sum(np.square(self.vector1 - self.vector2)))

    def euc_2(self):
        return np.linalg.norm(self.vector1 - self.vector2)

    def euc_norm(self):
        return np.linalg.norm(np_norm(self.vector1) - (self.vector2))



import concurrent.futures
import math

PRIMES = [
    112272535095293,
    112582705942171,
    112272535095293,
    115280095190773,
    115797848077099,
    1099726899285419]


def is_prime(n):
    if n < 2:
        return False
    if n == 2:
        return True
    if n % 2 == 0:
        return False

    sqrt_n = int(math.floor(math.sqrt(n)))
    for i in range(3, sqrt_n + 1, 2):
        if n % i == 0:
            return False
    return True


def main():
    # multi process
    with concurrent.futures.ProcessPoolExecutor(max_workers=None) as executor:
        for number, prime in zip(PRIMES, executor.map(is_prime, PRIMES)):
            print('%d is prime: %s' % (number, prime))
    # multi thread
    with concurrent.futures.ThreadPoolExecutor(max_workers=None) as executor:
        for number, prime in zip(PRIMES, executor.map(is_prime, PRIMES)):
            print('%d is prime: %s' % (number, prime))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# coding: utf-8

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='cv2box',  # 项目的名称,pip3 install get-time
    version='0.1.0',  # 项目版本
    author='ykk',  # 项目作者
    author_email='ykk648@gmail.com',  # 作者email
    url='https://github.com/ykk648/cv2box',  # 项目代码仓库
    project_urls={
        "Bug Tracker": "https://github.com/ykk648/cv2box/issues",
    },
    description='cv toolbox',  # 项目描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    # package_dir={"": "src"},
    packages=setuptools.find_packages(),
    install_requires=['numpy', 'tqdm', 'opencv_python', 'Pillow', 'matplotlib', 'seaborn', 'moviepy'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)



# from multiprocessing import Process, Lock, Queue # multi process
from multiprocessing.dummy import Process, Queue  # multi thread
import os
import time
import queue

try:
    from cv2box import flush_print as fp
except ModuleNotFoundError:
    fp = print


class Consumer(Process):
    def __init__(self, queue_list: list, block=True, fps_counter=False):
        super().__init__()
        self.queue_list = queue_list
        self.fps_counter = fps_counter
        self.block = block
        self.pid = os.getpid()

        # add init here

        print('init consumer {}, pid is {}.'.format(self.__class__.__name__, self.pid))

    def forward_func(self, something_in):

        # do your work here.
        something_out = something_in
        return something_out

    def run(self):

        counter = 0
        queue_full_counter = 0
        start_time = time.time()

        while True:
            something_in = self.queue_list[0].get()

            # exit condition
            if something_in is None:
                print('subprocess {} exit !'.format(self.pid))
                break

            something_out = self.forward_func(something_in)

            if self.block:
                self.queue_list[1].put(something_out)
            else:
                try:
                    self.queue_list[1].put_nowait(something_out)
                except queue.Full:
                    # do your judge here, for example
                    queue_full_counter += 1
                    if (time.time() - start_time) > 10:
                        fp('Queue full {} times'.format(queue_full_counter))

            if self.fps_counter:
                counter += 1
                if (time.time() - start_time) > 10:
                    fp("Consumer FPS: {}".format(counter / (time.time() - start_time)))
                    counter = 0
                    start_time = time.time()


if __name__ == '__main__':
    q1 = Queue(10)
    q2 = Queue(10)
    c1 = Consumer([q1, q2])
    c1.start()
